@@ -9,6 +9,8 @@
 #'
 #'
 #'   @param dset    the data frame for which cvi is needed
+#'   @param tset    [NULL] optional validation data set. If NULL, the validation set is build from
+#'                  \code{dset} according to the procedure prescribed in \code{opts$TST.*}. 
 #'   @param response.variables   name of column which carries the target variable - or -
 #'                   vector of names specifying multiple target columns
 #'                   (these columns are not used during prediction, only for evaluation)
@@ -29,8 +31,13 @@
 #'       \item{C_train}{ classification error on training set}
 #'       \item{G_train}{ gain on training set}
 #'       \item{R_train}{ relative gain on training set (percentage of max. gain on this set)}
-#'       \item{*_test}{ --- similar, with test set instead of training set}
-#'       \item{*_test2}{ --- similar, with test2 set instead of training set}
+#'       \item{*_test}{ --- similar, with test set instead of training set ---   }
+#'       \item{*_test2}{ --- similar, with test2 set instead of training set ---  }
+#'       \item{Err}{ }
+#'       \item{predictions}{ last run: data frame with dimensions [nrow(dset),length(response.variable)]. In case of CV, all 
+#'              CV predictions (for each record in dset), in other cases mixed validation / train set predictions.  }
+#'       \item{predProbList}{ the ith element in this list has data on the prediction probabilities of the ith run. See info on 
+#'              \code{predProb} in \code{\link{tdmClassify}}. }
 #'
 #'    Each performance measure \code{C_*, G_*, R_*} is a vector of length \code{opts$NRUN}. To be specific, \code{C_train[i]} is the
 #'    classification error on the training set from the \code{i}-th run. This error is \code{mean(res$allEVAL$cerr.trn)}, i.e. the
@@ -38,26 +45,30 @@
 #'    In the case of cross validation, for each performance measure an additional averaging over all folds is done.
 #'
 #' @seealso   \code{\link{print.TDMclassifier}}, \code{\link{tdmClassify}}, \code{\link{tdmRegress}}, \code{\link{tdmRegressLoop}}
-#' @author Wolfgang Konen (\email{wolfgang.konen@@fh-koeln.de}), FHK, Sep'2010 - Apr'2012
+#' @author Wolfgang Konen (\email{wolfgang.konen@@fh-koeln.de}), FHK, Sep'2010 - Jun'2012
 #' @aliases TDMclassifier 
 #' @example demo/demo00sonar.r
 #' @export
 ######################################################################################
-tdmClassifyLoop <- function(dset,response.variables,input.variables,opts) {
+tdmClassifyLoop <- function(dset,response.variables,input.variables,opts,tset=NULL) {
   	if (exists(".Random.seed")) SAVESEED<-.Random.seed	   #save the Random Number Generator RNG status
 #print(.Random.seed[1:6])
+    if (class(opts)[1] != "tdmOpts")  stop("Class of object opts is not tdmClass");
     if (is.null(opts$PRE.PCA.numericV)) opts$PRE.PCA.numericV <- input.variables;
+    if (opts$NRUN<=0) stop(sprintf("opts$NRUN has to be positive, but it is %d",opts$NRUN));
 
-    C_train <- C_test <- C_test2 <- G_train <- G_test <- NULL # reserve names (dynamic extension of
-    R_train <- R_test <- R_test2 <- G_test2 <- NULL           # these vectors at and of for-i-loop)
-
-    if (opts$READ.TST==TRUE & opts$TST.kind!="col")
-      warning(sprintf("Are you sure you want opts$READ.TST=TRUE, but opts$TST.kind!='col'? Actual value is opts$TST.kind='%s'.",opts$TST.kind));
+    #if (opts$READ.TST==TRUE & opts$TST.kind!="col")
+    #  warning(sprintf("Are you sure you want opts$READ.TST=TRUE, but opts$TST.kind!='col'? Actual value is opts$TST.kind='%s'.",opts$TST.kind));
 
     predProbList=list();
-    #predProbList=list();
-    
+    C_train <- C_test <- C_test2 <- G_train <- G_test <- NULL # reserve names (dynamic extension of
+    R_train <- R_test <- R_test2 <- G_test2 <- NULL           # these vectors at and of for-i-loop)
+    Err <- NULL    
     for (i in 1:opts$NRUN) {
+        if (opts$NRUN>1) {
+          if (opts$GD.RESTART) tdmGraphicCloseDev(opts);
+          tdmGraphicInit(opts);
+        }
         opts$i = i;
 
         #=============================================================================
@@ -79,26 +90,34 @@ tdmClassifyLoop <- function(dset,response.variables,input.variables,opts) {
 #print(.Random.seed[1:6])
 
         cvi <- tdmModCreateCVindex(dset,response.variables,opts,stratified=TRUE);    
-        nfold = max(cvi);
+        nfold = max(cvi,1);     # Why ',1'? - In the special case where all records belong to train sample (max(cvi)=0), we want 
+                                # to have exactly one fold  (one pass through k-loop) and NOT k in { 1, 0 }
 
 #print(cvi[1:20])
 #browser();
         #=============================================
         # PART 4: MODELING, EVALUATION, VISUALIZATION
         #=============================================
-        allerr = NULL;
+        alltrn = alltst = NULL;
+        predictions <- as.data.frame(dset[,response.variables]);
+        names(predictions) <- response.variables;
         predProb=list();
-        
         for (k in 1:nfold) {
             opts$k=k;
             opts$the.nfold = nfold;
             cat1(opts,"\n")
-            d_test  <- dset[cvi==k, ]            # test set
+            d_test  <- dset[cvi==k, ]            # validation set
             d_train <- dset[cvi!=k & cvi>=0, ]   # training set (disregard records with cvi<0)
             d_dis   <- dset[cvi!=k & cvi==-1, ]  # disregard set (needed for active learning)
             d_test  <- tdmBindResponse(d_test , "IND.dset", which(cvi==k));
             d_train <- tdmBindResponse(d_train, "IND.dset", which(cvi!=k & cvi>=0));
             d_dis   <- tdmBindResponse(d_dis  , "IND.dset", which(cvi!=k & cvi==-1));
+            if (!is.null(tset)) {    # validation set (test set) is passed in from the caller
+              d_test <- tset;
+              d_test  <- tdmBindResponse(d_test , "IND.dset", rep(-1,nrow(tset)));
+            }
+            ntst=nrow(d_test);
+            ntrn=nrow(d_train);
             #if (nrow(d_train)+nrow(d_test)+nrow(d_dis) != nrow(dset))
             #  stop("Something wrong, the union of d_train, d_test and d_dis does not cover dset");
  
@@ -107,8 +126,9 @@ tdmClassifyLoop <- function(dset,response.variables,input.variables,opts) {
               # b) add monomials of degree 2 for the first opts$PRE.PCA.npc numeric variables
               # c) apply this PCA and monomials to d_test and d_dis in the same way
               other.variables <- setdiff(input.variables,opts$PRE.PCA.numericV);
-              pca <- tdmPrePCA.train(d_train,opts);                 # see tdmPreprocUtils.r
-              d_train <- pca$dset;
+              d_preproc <- dset[cvi!=k, ]         # all non-validation-data are used for preproc
+              pca <- tdmPrePCA.train(d_preproc,opts);                 # see tdmPreprocUtils.r
+              d_train <- tdmPrePCA.apply(d_train,pca$pcaList,opts,d_train)$dset;
               d_test <- tdmPrePCA.apply(d_test,pca$pcaList,opts,d_train)$dset;
               d_dis <- tdmPrePCA.apply(d_dis,pca$pcaList,opts,d_train)$dset;
 
@@ -117,58 +137,72 @@ tdmClassifyLoop <- function(dset,response.variables,input.variables,opts) {
               if (length(setdiff(input.variables,names(d_train)))>0) 
                   stop("Some elements of input.variables are not columns of d_train");
             }
+            if (opts$PRE.SFA.npc>0 | opts$PRE.SFA!="none") {
+              d_preproc <- dset[cvi!=k, ]         # all non-validation-data are used for preproc
+            } else {
+              d_preproc <- NULL;
+            }
 
             # the SFA preprocessing is now in tdmClassify, inside response.variable for-loop
             # (because SFA depends on response.variable)
 
-            res <- tdmClassify(d_train,d_test,d_dis,response.variables,input.variables,opts)
+            res <- tdmClassify(d_train,d_test,d_dis,d_preproc,response.variables,input.variables,opts)
 
             # predProb$Val: bind the different folds of CV (cross validation) together. If no CV, then nfold=1 ->> only results from one vali set.
             predProb$Val = rbind(predProb$Val,res$predProb$Val);
             # predProb$Trn: take only the results from the first fold
             if (k==1) predProb$Trn = res$predProb$Trn;
             
-            allerr = rbind(allerr,as.data.frame(list(cerr.trn=mean(res$allEVAL$cerr.trn)
-                                                    ,gain.trn=mean(res$allEVAL$gain.trn)
-                                                    ,rgain.trn=mean(res$allEVAL$rgain.trn)
-                                                    ,cerr.tst=mean(res$allEVAL$cerr.tst)
-                                                    ,gain.tst=mean(res$allEVAL$gain.tst)
-                                                    ,rgain.tst=mean(res$allEVAL$rgain.tst)
-                                                    ,cerr.tst2=mean(res$allEVAL$cerr.tst2)
-                                                    ,gain.tst2=mean(res$allEVAL$gain.tst2)
-                                                    ,rgain.tst2=mean(res$allEVAL$rgain.tst2)
+            alltrn = rbind(alltrn,as.data.frame(list(cerr.trn=mean(res$allEVAL$cerr.trn) * ntrn
+                                                    ,gain.trn=mean(res$allEVAL$gain.trn) * ntrn
+                                                    ,rgain.trn=mean(res$allEVAL$rgain.trn) * ntrn
+                                                    ,ntrn=ntrn
                                                     )));
-#            # OLD VERSION until Dec'2011
-#            allerr = rbind(allerr,as.data.frame(list(cerr.trn=res$sumEVAL$cerr.trn
-#                                                    ,gain.trn=res$sumEVAL$gain.trn
-#                                                    ,rgain.trn=res$sumEVAL$rgain.trn
-#                                                    ,cerr.tst=res$sumEVAL$cerr.tst
-#                                                    ,gain.tst=res$sumEVAL$gain.tst
-#                                                    ,rgain.tst=res$sumEVAL$rgain.tst
-#                                                    ,cerr.tst2=res$sumEVAL$cerr.tst2
-#                                                    ,gain.tst2=res$sumEVAL$gain.tst2
-#                                                    ,rgain.tst2=res$sumEVAL$rgain.tst2
-#                                                    )));
+            alltst = rbind(alltst,as.data.frame(list(cerr.tst=mean(res$allEVAL$cerr.tst) * ntst
+                                                    ,gain.tst=mean(res$allEVAL$gain.tst) * ntst
+                                                    ,rgain.tst=mean(res$allEVAL$rgain.tst) * ntst
+                                                    ,cerr.tst2=mean(res$allEVAL$cerr.tst2) * ntst
+                                                    ,gain.tst2=mean(res$allEVAL$gain.tst2) * ntst
+                                                    ,rgain.tst2=mean(res$allEVAL$rgain.tst2)
+                                                    ,ntst=ntst
+                                                    )));
+            if (is.null(tset)) {
+              predictions[cvi==k,response.variables] <- res$d_test[,paste("pred_",response.variables,sep="")];
+              predictTest = NULL;
+            } else {
+              predictTest = res$d_test[,paste("pred_",response.variables,sep="")];
+            }
+            if (!(opts$TST.kind=="cv")) {
+              predictions[cvi!=k & cvi>=0,response.variables] <- res$d_train[1:ntrn,paste("pred_",response.variables,sep="")];
+                                                                     # Why '1:ntrn'? - If opts$ncopies>0, then tdmClassify has added
+                                                                     # opts$ncopies bootstrap patterns to the ntrn original patterns.
+                                                                     # These bootstrap patterns have to be discarded here.
+            } 
         }  # for (k in 1:nfold)
 
-        Err = colSums(allerr)/nfold     # assumes that each fold has the same (or nearly the same) size
+        ### -- OLD: Err = colSums(allerr)/nfold     # assumes that each fold has the same (or nearly the same) size
+        # each measure is a weighted average over folds, weighted with #cases in each fold:
+        Err = rbind(Err,c(colSums(alltrn)/sum(alltrn$ntrn),colSums(alltst)/sum(alltst$ntst)));
+        Err[i,"ntrn"]=ifelse(opts$TST.kind=="cv",nrow(dset),nrow(d_train));   # the number of distinct cases 
+        Err[i,"ntst"]=ifelse(opts$TST.kind=="cv",nrow(dset),nrow(d_test));    # used for this line of measures
+
         predProbList[[i]] <- list()
         predProbList[[i]]$Val <- predProb$Val;
         predProbList[[i]]$Trn <- predProb$Trn;
 
-        cat1(opts,"\n",ifelse(opts$TST.kind=="cv","CV",""),"Relative gain on training set   ",Err["rgain.trn"],"%\n")
-        cat1(opts,"",  ifelse(opts$TST.kind=="cv","CV",""),"Relative gain on     test set   ",Err["rgain.tst"],"%\n\n")
+        cat1(opts,"\n",ifelse(opts$TST.kind=="cv","CV",""),"Relative gain on   training set   ",Err[i,"rgain.trn"],"%\n")
+        cat1(opts,"",  ifelse(opts$TST.kind=="cv","CV",""),"Relative gain on validation set   ",Err[i,"rgain.tst"],"%\n\n")
 
         #opts$name.of.prediction <- paste("pred_", response.variable, sep="")
-        C_train[i] = Err["cerr.trn"]
-        G_train[i] = Err["gain.trn"]
-        R_train[i] = Err["rgain.trn"]
-        C_test[i] = Err["cerr.tst"]
-        G_test[i] = Err["gain.tst"]
-        R_test[i] = Err["rgain.tst"]
-        C_test2[i] = Err["cerr.tst2"]
-        G_test2[i] = Err["gain.tst2"]             # for comparision in case of opts$MOD.method="RF": results with
-        R_test2[i] = Err["rgain.tst2"]            # default cutoff 1/n.class instead of opts$CLS.cutoff
+        C_train[i] = Err[i,"cerr.trn"]
+        G_train[i] = Err[i,"gain.trn"]
+        R_train[i] = Err[i,"rgain.trn"]
+        C_test[i] = Err[i,"cerr.tst"]
+        G_test[i] = Err[i,"gain.tst"]
+        R_test[i] = Err[i,"rgain.tst"]
+        C_test2[i] = Err[i,"cerr.tst2"]
+        G_test2[i] = Err[i,"gain.tst2"]             # for comparision in case of opts$MOD.method="RF": results with
+        R_test2[i] = Err[i,"rgain.tst2"]            # default cutoff 1/n.class instead of opts$CLS.cutoff
 
         #=============================================
         # PART 5: SOME MORE GRAPHICS
@@ -205,7 +239,11 @@ tdmClassifyLoop <- function(dset,response.variables,input.variables,opts) {
                 	, C_test2 = C_test2
                 	, G_test2 = G_test2
                 	, R_test2 = R_test2
-                	, predProbList = predProbList
+                	, Err = Err
+                	, predictions = predictions     # predictions on dset from the *last* run: all CV predictions in case of CV, 
+                                                  # mixed vali-train predictions else
+                  , predictTest = predictTest
+                	, predProbList = predProbList   # ith element: a list with predProb data from the ith run 
                 	);
     if (!is.null(opts$TST.COL))         # needed when result is input for coTraining  (see ssl_methods.r)
       if (opts$TST.COL %in% names(dset))  result$TST = dset[,opts$TST.COL]
@@ -275,11 +313,11 @@ tdmClassifySummary <- function(result,opts,dset=NULL)
       result$y=-y;             # the score (to be minimized by SPOT) is "minus test score"
       result$sd.y=sd(result$R_test);
     }
-    cat1(opts,sprintf("%s  Test    relative gain: %7.3f",ifelse(opts$TST.kind=="cv","CV ",""),y));
+    cat1(opts,sprintf("%s  Vali    relative gain: %7.3f",ifelse(opts$TST.kind=="cv","CV ",""),y));
     cat1(opts,ifelse(opts$NRUN>1,sprintf(" +-%7.3f",sd(result$R_test)),""));
     cat1Records(nrow(res$d_test));
 
-    cat1(opts,sprintf("%s Test2    relative gain (predict always with %s): %7.3f"
+    cat1(opts,sprintf("%s Vali2    relative gain (predict always with %s): %7.3f"
                      ,ifelse(opts$TST.kind=="cv","CV ",""), res$allEVAL$test2.string, mean(result$R_test2)));
     cat1(opts,ifelse(opts$NRUN>1,sprintf(" +-%7.3f",sd(result$R_test2)),""));
     cat1Records(nrow(res$d_test));
