@@ -21,6 +21,7 @@ tdm <- list( mainFunction="main_sonar"
             , nrun=5, nfold=2         # repeats and CV-folds for the unbiased runs
             , withParams=TRUE         # list the columns with tuned parameter in final results 
             , nExperim=2
+            , parallelCPUs=2
             , optsVerbosity = 0       # the verbosity for the unbiased runs
             );
 ## fill in other defaults for list tdm            
@@ -28,10 +29,27 @@ tdm <- tdmDefaultsFill(tdm);
 ## sonar_05.conf has the settings for the tuning process (e.g. "auto.loop.steps"=number of SPOT generations       
 ## "auto.loop.evals"=budget of model building runs and io.roiFileName = "sonar_04.roi").
 ## runList could contain other files as well (e.g. c("sonar_01.conf","sonar_02.conf","sonar_03.conf")), if desired.
-runList = c("sonar_05.conf");     
+tdm$runList = c("sonar_05.conf");     
 ## spotStep can be either "auto" (do automatic tuning) or "rep" (make a visual report of best results)
 spotStep = "auto";
 
+## construct an initial environment envT from the given TDMR settings in tdm
+## (this contains also the fill-in of other defaults for tdm via
+##      envT$tdm <- tdmDefaultsFill(tdm);
+## )
+envT <- tdmEnvTMakeNew(tdm);
+
+## the call to tdmBigLoop will start the whole TDMR process:
+## - for each file in tdm$runList a complete DM tuning is started with each tuning method tdm$tuneMethod  (if spotStep=="auto")
+## - the best result from tuning is fed into an unbiased model build and evaluation run 
+## - results are printed and returned in envT$theFinals 
+## - more detailed results are in other elements of environment envT
+## - two plots: 
+##      a) the progression of the response variable Y and the parameter variables during tuning
+##      b) the sensitivity plot for each parameter in the vicinity of the best solution found 
+envT <- tdmBigLoop(envT,spotStep);
+
+#---- deprecated ----
 ## the call to tdmCompleteEval will start the whole TDMR process:
 ## - for each file in runList a complete DM tuning is started with each tuning method
 ## - the best result from tuning are fed into an unbiased model building and evaluation run 
@@ -40,7 +58,8 @@ spotStep = "auto";
 ## - two plots: 
 ##      a) the progression of the response variable Y and the parameter variables during tuning
 ##      b) the sensitivity plot for each parameter in the vicinity of the best solution found 
-envT <- tdmCompleteEval(runList,NULL,spotStep,tdm);
+#envT <- tdmCompleteEval(runList,NULL,spotStep,tdm);
+#---- deprecated ----
 
 ## restore old working directory
 setwd(oldwd);
