@@ -6,23 +6,25 @@
 #
 #     Additionally, parallel execution (snowfall) is initialized according to the settings in tdm
 #
+#     start.tdm.r is not needed for the normal TDMR user (who loads the library TDMR). It is only
+#     needed for the TDMR developer who wants to start TDMR from R sources (with or w/o parallel execution)
+#
 ###############################################################################################
 
 #
 # script part to initialize parallel execution if tdm$parallelCPUs>1
+# (this part is now normally done in tdmBigLoop, it is only needed here in case of a 'developer start' (from R sources)
 #
-#if (is.na(match("USE.SPOT.PACKAGE",ls()))) USE.SPOT.PACKAGE <<- TRUE;   # note "<<-" ! USE.SPOT.PACKAGE needs to be def'd in .GlobalEnv
 if (is.na(match("tdm",ls()))) tdm <- list();
 if (is.null(tdm$theSpotPath)) tdm$theSpotPath <- NA;
 if (is.null(tdm$theRsfaPath)) tdm$theRsfaPath <- NA;
 if (is.null(tdm$parallelCPUs)) tdm$parallelCPUs=1;
-if (tdm$parallelCPUs>1) {
-  require(snow)
-  require(snowfall)
-  parallelCPUs = tdm$parallelCPUs;
-  sfInit(parallel=TRUE, cpus=parallelCPUs, type="SOCK" )
-#  sfExport(list=c("USE.SPOT.PACKAGE"))
-  sfExport(list=c("tdm"))
+if (tdm$parallelCPUs>1) {               
+    require(snow)
+    require(snowfall)
+    parallelCPUs = tdm$parallelCPUs;
+    sfInit(parallel=TRUE, cpus=parallelCPUs, type="SOCK" )
+    sfExport(list=c("tdm"))
 }
 
 #
@@ -30,11 +32,11 @@ if (tdm$parallelCPUs>1) {
 #
 if (is.null(tdm$tdmPath)) {
     cat("Loading installed library TDMR \n");
-    require(TDMR)
+    require(TDMR);
+    if (tdm$parallelCPUs>1) sfLibrary(TDMR);
 } else {
     cat("Sourcing TDMR from R files in",tdm$tdmPath,"\n");
     source(paste(tdm$tdmPath,"source.tdm.r",sep="/"),local=TRUE);
     source.tdm(tdm$tdmPath,tdmParallelCPUs=tdm$parallelCPUs,theSpotPath=tdm$theSpotPath,theRsfaPath=tdm$theRsfaPath);
+    if (tdm$parallelCPUs>1) sfExport(list=c("tdmRandomSeed"));
 }
-#tdmRandomSeed <- makeTdmRandomSeed();      # this is now in makeTdmRandomSeed.r
-if (tdm$parallelCPUs>1) sfExport(list=c("tdmRandomSeed"));
